/*
 * JZ4730 CPM server.
 *
 * Copyright (C) 2018 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <l4/devices/cpm-jz4730.h>
#include <l4/devices/memory.h>
#include "cpm-server.h"

#include <l4/re/dataspace>
#include <l4/re/env>
#include <l4/re/util/object_registry>

#include <l4/sys/types.h>

// Virtual addresses for the CPM and LCD register blocks.

static l4_addr_t cpm_virt_base = 0, cpm_virt_base_end = 0;



// Access to peripheral memory.

static int setup_memory()
{
  if (get_memory("jz4730-cpm", &cpm_virt_base, &cpm_virt_base_end))
    return 1;

  return 0;
}



static L4Re::Util::Registry_server<> server;



/* Main program. */

int main(void)
{
  if (setup_memory()) return 1;

  /* Initialise the CPM abstraction. */

  Cpm_jz4730_chip cpm_device(cpm_virt_base, 3686400);

  /* Initialise and register a server object. */

  Cpm_server server_obj(&cpm_device);
  server.registry()->register_obj(&server_obj, "cpm");

  /* Enter the IPC server loop. */

  server.loop();
  return 0;
}
