/*
 * Common CPM server functionality.
 *
 * Copyright (C) 2018 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include "cpm-client.h"
#include "cpm-server.h"
#include "cpm-ops.h"

#include <l4/cxx/ipc_server>
#include <l4/re/env>
#include <l4/sys/typeinfo_svr>
#include <l4/util/util.h>

/* Handle invocations. */

int
Cpm_server::dispatch(l4_umword_t obj, L4::Ipc::Iostream &ios)
{
  l4_msgtag_t tag;

  (void) obj;
  ios >> tag;

  switch (tag.label())
  {
    case L4::Meta::Protocol:
      return L4::Util::handle_meta_request<Cpm_device_interface>(ios);

    case Cpm_op_set_lcd_frequencies:
      uint32_t pclk;
      int multiplier;
      ios >> pclk;
      ios >> multiplier;
      _chip->set_lcd_frequencies(pclk, multiplier);
      return L4_EOK;

    case Cpm_op_start_lcd:
      _chip->start_lcd();
      return L4_EOK;

    case Cpm_op_stop_lcd:
      _chip->stop_lcd();
      return L4_EOK;

    case Cpm_op_update_output_frequency:
      _chip->update_output_frequency();
      return L4_EOK;

    default:
      return -L4_EBADPROTO;
  }
}
