/*
 * Common framebuffer server functionality.
 *
 * Copyright (C) 2018 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#ifdef __cplusplus

#include <l4/cxx/ipc_server>
#include <l4/re/dataspace>
#include <l4/re/env>
#include <l4/re/c/video/view.h>
#include <l4/sys/types.h>

/* Server object to provide framebuffer data access. */

class Framebuffer_server : public L4::Server_object_t<L4::Kobject>
{
private:
  L4::Cap<L4Re::Dataspace> _mem;

public:
  /* Initialise the server with a capability referencing the exported memory. */

  explicit Framebuffer_server(L4::Cap<L4Re::Dataspace> mem)
  : _mem(mem)
  {}

  int dispatch(l4_umword_t obj, L4::Ipc::Iostream &ios);

  /* Operation methods. */

  virtual l4_size_t get_framebuffer_size() = 0;
  virtual void get_view_info(l4re_video_view_info_t *view_info) = 0;
};

#endif
