/*
 * Framebuffer client to access framebuffer servers.
 *
 * Copyright (C) 2018 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include "fb-client.h"
#include "fb-ops.h"

#include <l4/cxx/ipc_stream>
#include <l4/re/env>

#include <stdint.h>

int
Framebuffer_device_interface::get_framebuffer(L4::Cap<L4Re::Dataspace> mem) throw()
{
  L4::Ipc::Iostream s(l4_utcb());

  /* Send a "receive item" for requesting a capability. */

  s << L4::Ipc::Small_buf(mem);

  return l4_error(s.call(cap(), Framebuffer_op_get_framebuffer));
}

l4_size_t
Framebuffer_device_interface::get_framebuffer_size() throw()
{
  L4::Ipc::Iostream s(l4_utcb());
  l4_size_t size;

  /* Return zero on failure. */

  if (l4_error(s.call(cap(), Framebuffer_op_get_framebuffer_size)))
    return 0;

  s >> size;
  return size;
}

int
Framebuffer_device_interface::get_view_info(l4re_video_view_info_t *view_info) throw()
{
  L4::Ipc::Iostream s(l4_utcb());

  int err = l4_error(s.call(cap(), Framebuffer_op_get_view_info));
  if (err) return err;

  unsigned long expected = sizeof(*view_info);
  unsigned long size = expected;

  s >> L4::Ipc::buf_cp_in<uint8_t>((uint8_t *) view_info, expected);

  if (size != expected) return -L4_EIO;
  return L4_EOK;
}
