/*
 * Export the framebuffer as a data space accessible via the "fb" capability.
 *
 * Copyright (C) 2018 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <l4/devices/lcd-device.h>
#include "fb-lcd.h"
#include "fb-server.h"

#include <l4/cxx/ipc_stream>
#include <l4/re/dataspace>
#include <l4/re/env>
#include <l4/re/util/object_registry>



// Device-specific methods.

l4_size_t
Framebuffer_lcd_server::get_framebuffer_size()
{
  return _device->get_framebuffer_size();
}

void
Framebuffer_lcd_server::get_view_info(l4re_video_view_info_t *view_info)
{
  _device->get_view_info(view_info);
}



static L4Re::Util::Registry_server<> server;



// Main program.

int run()
{
  // Obtain the LCD device.

  Lcd_device *lcd_device = Lcd_device::get_device();

  // Memory allocation capability for the framebuffer data.

  L4::Cap<L4Re::Dataspace> mem = lcd_device->get_framebuffer_cap();

  if (!mem.is_valid()) return 1;

  // Enable the LCD device.

  lcd_device->enable();

  // Initialise and register a server object.

  Framebuffer_lcd_server server_obj(mem, lcd_device);
  server.registry()->register_obj(&server_obj, "fb");

  // Enter the IPC server loop.

  server.loop();
  return 0;
}
