/*
 * Common framebuffer server functionality.
 *
 * Copyright (C) 2018 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include "fb-client.h"
#include "fb-server.h"
#include "fb-ops.h"

#include <l4/cxx/ipc_server>
#include <l4/re/env>
#include <l4/sys/typeinfo_svr>
#include <l4/util/util.h>

#include <stdint.h>

/* Handle invocations. */

int
Framebuffer_server::dispatch(l4_umword_t obj, L4::Ipc::Iostream &ios)
{
  l4_msgtag_t tag;

  (void) obj;
  ios >> tag;

  switch (tag.label())
  {
    case L4::Meta::Protocol:
      return L4::Util::handle_meta_request<Framebuffer_device_interface>(ios);

    case Framebuffer_op_get_framebuffer:
      ios << _mem;
      return L4_EOK;

    case Framebuffer_op_get_framebuffer_size:
      ios << get_framebuffer_size();
      return L4_EOK;

    case Framebuffer_op_get_view_info:
      l4re_video_view_info_t view_info;
      get_view_info(&view_info);

      /* Serialise the view information structure. */

      ios << L4::Ipc::buf_cp_out<uint8_t>((uint8_t *) &view_info, sizeof(view_info));
      return L4_EOK;

    default:
      return -L4_EBADPROTO;
  }
}
