/*
 * Input event loop functionality.
 *
 * Copyright (C) 2018 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#ifdef __cplusplus

#include <l4/devices/event-loop.h>

#include <l4/re/event>
#include <l4/sys/capability>
#include <l4/sys/icu>

/* Input event loop abstraction. */

class Input_event_loop : public Event_handler_loop<L4Re::Event_buffer::Event>
{
  /* Event buffer and notification interrupt capability. */

  L4Re::Event_buffer _event_buffer;

  /* Notification interrupt capability. */

  L4::Cap<L4::Irq> _irq;

public:
  /* Initialise the event loop with an event buffer, a notification interrupt,
     and a thread priority. */

  explicit Input_event_loop(L4Re::Event_buffer event_buffer,
                            L4::Cap<L4::Irq> irq,
                            int priority=0x20)
  : Event_handler_loop(priority), _event_buffer(event_buffer), _irq(irq)
  {
  }

  /* Event handler method, dispatching to the provided handler. */

  virtual void handle();

  /* Start the loop, dispatching to the handle method. */

  virtual void start(l4_umword_t label=0xDF00);
};

#endif
