/*
 * Common input event server functionality.
 *
 * Copyright (C) 2018 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#ifdef __cplusplus

#include "input-event-client.h"

#include <l4/cxx/ipc_server>
#include <l4/re/dataspace>
#include <l4/re/env>
#include <l4/re/event>
#include <l4/sys/icu>
#include <l4/sys/types.h>

/* Server object to provide input event source access. */

class Input_event_server : public L4::Server_object_t<Input_event_interface>
{
private:
  L4::Cap<L4Re::Dataspace> _mem;
  L4::Cap<L4::Irq> _irq;
  L4Re::Event_buffer _events;

public:
  /* Initialise the server with a capability referencing the exported memory
     and an event buffer through which events will be communicated. */

  explicit Input_event_server(L4::Cap<L4Re::Dataspace> mem,
                              L4Re::Event_buffer events)
  : _mem(mem), _events(events)
  {
  }

  int dispatch(l4_umword_t obj, L4::Ipc::Iostream &ios);

  /* Event sending methods. */

  void send_event(int type, int code, int value);
};

#endif

int run(void);
