/*
 * Common keypad client functionality for input event generation.
 *
 * Copyright (C) 2018 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include <l4/devices/keypad.h>
#include <l4/devices/keypad-client.h>

#include <pthread.h>
#include <pthread-l4.h>

#include <l4/re/event_enums.h>

#include <stdint.h>

/* Input event and handler. */

typedef struct
{
  enum L4Re_events_ev type;
  int code;
  int value;
} Input_event;

typedef void (*Input_handler)(Input_event, void *);



#ifdef __cplusplus

#include <l4/re/dataspace>
#include <l4/re/env>

/* Client object to provide input event access. */

class Input_keypad_client
{
  /* Memory and keypad access capabilities. */

  L4::Cap<L4Re::Dataspace> _mem;
  L4::Cap<Keypad_device_interface> _keypad_server;

  /* Keypad layout details. */

  Keypad *_keypad;

  /* Keypad memory. */

  void *_keymem = 0, *_keymem_previous = 0;

  /* Input framework integration. */

  Input_handler _handler = 0;

  /* Thread details. */

  pthread_t _pthread;
  void *_priv;

  /* Initialisation functions. */

  void init_memory();
  void init_keypad();
  void init_keypad_data();
  void release_keypad_data();

  /* Activity methods. */

  static void *scan_mainloop(void *data);
  void scan_keypad();

public:
  explicit Input_keypad_client(Keypad *keypad) : _keypad(keypad)
  {
    init_memory();
    init_keypad();
    init_keypad_data();
  }

  ~Input_keypad_client()
  {
    release_keypad_data();
  }

  /* Thread initialisation. */

  void attach(Input_handler handler, void *priv);
};

#endif
