/*
 * Input event client to access input event servers. This is a more narrow
 * alternative to the L4Re::Event interface.
 *
 * Copyright (C) 2018 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <l4/cxx/ipc_stream>
#include <l4/re/env>

#include "input-event-client.h"
#include "input-event-ops.h"

int
Input_event_interface::get_buffer(L4::Cap<L4Re::Dataspace> mem) throw()
{
  L4::Ipc::Iostream s(l4_utcb());

  s << Input_event_op_get_buffer;

  /* Send a "receive item" for requesting a capability. */

  s << L4::Ipc::Small_buf(mem);

  return l4_error(s.call(cap(), Input_event_proto_event));
}

int
Input_event_interface::bind(unsigned irqnum, L4::Cap<L4::Irq> irq) throw()
{
  L4::Ipc::Iostream s(l4_utcb());

  s << Input_event_op_bind;
  s << irqnum;
  s << irq;

  return l4_error(s.call(cap(), Input_event_proto_icu));
}
