/*
 * Input event loop functionality.
 *
 * Copyright (C) 2018 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include "input-event-loop.h"

#include <l4/re/event>

/* Handler for incoming interrupts signalling the presence of events. */

void
Input_event_loop::handle()
{
  Event_type *event;

  if (!l4_error(_irq->receive()))
  {
    while ((event = _event_buffer.next()))
    {
      _handler(*event);
      event->free();
    }
  }
}

/* Event loop initiation. */

void
Input_event_loop::start(l4_umword_t label)
{
  Event_loop::start();

  _irq->bind_thread(Pthread::L4::cap(_pthread), label);
}
