/*
 * LCD device support.
 *
 * Copyright (C) 2018 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include <l4/devices/display-client.h>
#include <l4/devices/lcd.h>

#include <l4/re/c/video/view.h>
#include <l4/sys/types.h>



#ifdef __cplusplus

#include <l4/re/dataspace>
#include <l4/sys/capability>

class Lcd_device
{
protected:
  /* LCD peripheral abstraction. */

  Lcd_chip *_chip;

  /* Display server abstraction. */

  L4::Cap<Display_device_interface> _display;

  /* Framebuffer virtual and physical addresses. */

  l4_addr_t fb_vaddr, fb_paddr;

  /* Memory capability for the framebuffer. */

  L4::Cap<L4Re::Dataspace> _fbmem;

  /* Display operations. */

  virtual void disable_display();
  virtual void enable_display();

public:
  /* Initialise a device with a controller and display object reference. */

  Lcd_device(Lcd_chip *chip, L4::Cap<Display_device_interface> display)
  : _chip(chip), _display(display)
  {
    /* Subclasses must set up any memory. */
  }

  /* Framebuffer operations. */

  virtual l4_addr_t get_framebuffer()
  {
    return fb_vaddr;
  }

  virtual L4::Cap<L4Re::Dataspace> get_framebuffer_cap()
  {
    return _fbmem;
  }

  /* Querying operations. */

  virtual l4_size_t get_framebuffer_size() = 0;
  virtual void get_view_info(l4re_video_view_info_t *view_info) = 0;

  /* Device operations. */

  virtual void disable() = 0;
  virtual void enable() = 0;

  /* Device access. */

  static Lcd_device *get_device();
};

#endif



/* C language interface. */

EXTERN_C_BEGIN

void *lcd_get_device(void);
l4_addr_t lcd_get_framebuffer(void *lcd);
l4_size_t lcd_get_framebuffer_size(void *lcd);
void lcd_get_view_info(void *lcd, l4re_video_view_info_t *view_info);
void lcd_disable(void *lcd);
void lcd_enable(void *lcd);

EXTERN_C_END
