/*
 * Copyright (C) 2017, 2018 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include "cpm.h"

#include <l4/sys/types.h>
#include <stdint.h>



#ifdef __cplusplus

#include <l4/devices/hw_register_block.h>

/* A simple abstraction for accessing the CPM registers.
 * A proper device could inherit from Hw::Device and use an
 * Int_property for _exclk_freq. */

class Cpm_jz4740_chip : public Cpm_chip
{
private:
  Hw::Register_block<32> _regs;
  uint32_t _exclk_freq;

  int pll_enabled();
  int pll_bypassed();

public:
  Cpm_jz4740_chip(l4_addr_t addr, uint32_t exclk_freq);

  int have_clock();
  void start_clock();

  int have_pll();

  uint16_t get_multiplier();
  uint8_t get_input_division();
  uint8_t get_output_division();

  uint8_t _get_divider(uint32_t reg, uint32_t mask, uint8_t shift);
  uint8_t get_cpu_divider();
  uint8_t get_hclock_divider();
  uint8_t get_pclock_divider();
  uint8_t get_memory_divider();
  uint8_t get_source_divider();

  uint16_t get_lcd_pixel_divider();
  uint32_t get_lcd_pixel_frequency();

  void set_lcd_device_divider(uint8_t division);
  void set_lcd_pixel_divider(uint16_t division);
  void set_lcd_frequencies(uint32_t pclk, uint8_t ratio);

  void start_lcd();
  void stop_lcd();

  uint32_t get_pll_frequency();
  uint32_t get_output_frequency();
  void update_output_frequency();

  uint32_t get_cpu_frequency();
  uint32_t get_hclock_frequency();
  uint32_t get_pclock_frequency();
  uint32_t get_memory_frequency();
};

#endif /* __cplusplus */



/* C language interface. */

EXTERN_C_BEGIN

void *jz4740_cpm_init(l4_addr_t cpm_base);

int jz4740_cpm_have_pll(void *cpm);

int jz4740_cpm_have_clock(void *cpm);
void jz4740_cpm_start_clock(void *cpm);

void jz4740_cpm_start_lcd(void *cpm);
void jz4740_cpm_stop_lcd(void *cpm);

uint16_t jz4740_cpm_get_lcd_pixel_divider(void *cpm);
uint32_t jz4740_cpm_get_lcd_pixel_frequency(void *cpm);

void jz4740_cpm_set_lcd_frequencies(void *cpm, uint32_t pclk, uint8_t ratio);
void jz4740_cpm_update_output_frequency(void *cpm);

uint32_t jz4740_cpm_get_cpu_frequency(void *cpm);
uint32_t jz4740_cpm_get_hclock_frequency(void *cpm);
uint32_t jz4740_cpm_get_output_frequency(void *cpm);
uint32_t jz4740_cpm_get_pclock_frequency(void *cpm);
uint32_t jz4740_cpm_get_memory_frequency(void *cpm);

EXTERN_C_END
