/*
 * Letux 400 key mapping.
 *
 * Copyright (C) 2018 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include "keypad.h"
#include <l4/re/event_enums.h>

enum Jz4730_keypad_gpio
{
  Jz4730_keypad_gpio_inputs_count = 8,
  Jz4730_keypad_gpio_outputs_count = 17,
};

class Keypad_letux400 : public Keypad
{
  /* Keycode definitions. */

  const int keycodes[Jz4730_keypad_gpio_inputs_count][Jz4730_keypad_gpio_outputs_count] = {

    {L4RE_KEY_PAUSE, L4RE_KEY_Q, L4RE_KEY_W, L4RE_KEY_E, L4RE_KEY_R, L4RE_KEY_U,
     L4RE_KEY_I, L4RE_KEY_O, 0, 0, 0, L4RE_KEY_P, 0, 0, 0, 0},

    {0, L4RE_KEY_TAB, L4RE_KEY_CAPSLOCK, L4RE_KEY_F3, L4RE_KEY_T, L4RE_KEY_Y,
     L4RE_KEY_RIGHTBRACE, L4RE_KEY_F7, 0, L4RE_KEY_BACKSPACE, 0,
     L4RE_KEY_LEFTBRACE, L4RE_KEY_SLEEP, 0, 0, 0, L4RE_KEY_LEFTSHIFT},

    {0, L4RE_KEY_A, L4RE_KEY_S, L4RE_KEY_D, L4RE_KEY_F, L4RE_KEY_J, L4RE_KEY_K,
     L4RE_KEY_L, 0, 0, 0, L4RE_KEY_SEMICOLON, 0, 0, 0, L4RE_KEY_UP,
     L4RE_KEY_RIGHTSHIFT},

    {0, L4RE_KEY_ESC, L4RE_KEY_BACKSLASH /* left */, L4RE_KEY_F4, L4RE_KEY_G,
     L4RE_KEY_H, L4RE_KEY_F6, 0, L4RE_KEY_SPACE, 0, L4RE_KEY_LEFTALT,
     L4RE_KEY_APOSTROPHE, 0, 0, 0, L4RE_KEY_DOWN, 0},

    {0, L4RE_KEY_Z, L4RE_KEY_X, L4RE_KEY_C, L4RE_KEY_V, L4RE_KEY_M,
     L4RE_KEY_COMMA, L4RE_KEY_DOT, L4RE_KEY_NUMLOCK, L4RE_KEY_ENTER, 0,
     L4RE_KEY_BACKSLASH /* right */, 0, 0, 0, L4RE_KEY_LEFT, 0},

    {0, 0, 0, 0, L4RE_KEY_B, L4RE_KEY_N, 0, L4RE_KEY_MENU,
     0, 0, 0, L4RE_KEY_SLASH, 0, 0, 0, L4RE_KEY_RIGHT, 0},

    {L4RE_KEY_LEFTCTRL, L4RE_KEY_GRAVE, 0, 0, L4RE_KEY_5, L4RE_KEY_6,
     L4RE_KEY_EQUAL, L4RE_KEY_F8, L4RE_KEY_DELETE, L4RE_KEY_F9, 0,
     L4RE_KEY_MINUS, 0, L4RE_KEY_F2, L4RE_KEY_INSERT, 0, L4RE_KEY_F1},

    {L4RE_KEY_F5, L4RE_KEY_1, L4RE_KEY_2, L4RE_KEY_3, L4RE_KEY_4, L4RE_KEY_7,
     L4RE_KEY_8, L4RE_KEY_9, 0, 0, L4RE_KEY_SYSRQ, L4RE_KEY_0, L4RE_KEY_F10, 0,
     0, 0, L4RE_KEY_FN}
  };

public:

  /* Number of keypad columns. */

  int columns()
  {
    return Jz4730_keypad_gpio_outputs_count;
  }

  /* Number of keypad rows, used by each value in the column array. */

  int rows()
  {
    return Jz4730_keypad_gpio_inputs_count;
  }

  /* Keycode accessors. */

  int code(int column, int row)
  {
    return keycodes[row][column];
  }

  /* Common instance. */

  static Keypad_letux400 keypad;
};

/* The actual definition. */

Keypad_letux400 Keypad_letux400::keypad;

/* Definition access. */

Keypad *Keypad::get_keypad()
{
  return &Keypad_letux400::keypad;
}
