/*
 * Ben NanoNote key mapping.
 *
 * Copyright (C) 2018 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include "keypad.h"
#include <l4/re/event_enums.h>

enum Jz4740_keypad_gpio
{
  Jz4740_keypad_gpio_inputs_count = 8,
  Jz4740_keypad_gpio_outputs_count = 8,
};

class Keypad_qi_lb60 : public Keypad
{
  /* Keycode definitions. */

  const int keycodes[Jz4740_keypad_gpio_inputs_count][Jz4740_keypad_gpio_outputs_count] = {

    {L4RE_KEY_F1, L4RE_KEY_F2, L4RE_KEY_F3, L4RE_KEY_F4, L4RE_KEY_F5,
     L4RE_KEY_F6, L4RE_KEY_F7, 0},

    {L4RE_KEY_Q, L4RE_KEY_W, L4RE_KEY_E, L4RE_KEY_R, L4RE_KEY_T, L4RE_KEY_Y,
     L4RE_KEY_U, L4RE_KEY_I},

    {L4RE_KEY_A, L4RE_KEY_S, L4RE_KEY_D, L4RE_KEY_F, L4RE_KEY_G, L4RE_KEY_H,
     L4RE_KEY_J, L4RE_KEY_K},

    {L4RE_KEY_ESC, L4RE_KEY_Z, L4RE_KEY_X, L4RE_KEY_C, L4RE_KEY_V, L4RE_KEY_B,
     L4RE_KEY_N, L4RE_KEY_M},

    {L4RE_KEY_TAB, L4RE_KEY_CAPSLOCK /* D-Sh */, L4RE_KEY_BACKSLASH,
     L4RE_KEY_APOSTROPHE, L4RE_KEY_COMMA, L4RE_KEY_DOT, L4RE_KEY_SLASH,
     L4RE_KEY_UP},

    {L4RE_KEY_O, L4RE_KEY_L, L4RE_KEY_EQUAL, L4RE_KEY_RIGHTALT /* Sym */,
     L4RE_KEY_SPACE, L4RE_KEY_F13 /* Qi */, L4RE_KEY_RIGHTCTRL, L4RE_KEY_LEFT},

    {L4RE_KEY_F8, L4RE_KEY_P, L4RE_KEY_BACKSPACE, L4RE_KEY_ENTER,
     L4RE_KEY_VOLUMEUP, L4RE_KEY_VOLUMEDOWN, L4RE_KEY_DOWN, L4RE_KEY_RIGHT},

    {L4RE_KEY_LEFTSHIFT, L4RE_KEY_LEFTALT, L4RE_KEY_FN, 0, 0, 0, 0, 0}
  };

public:

  /* Number of keypad columns. */

  int columns()
  {
    return Jz4740_keypad_gpio_outputs_count;
  }

  /* Number of keypad rows, used by each value in the column array. */

  int rows()
  {
    return Jz4740_keypad_gpio_inputs_count;
  }

  /* Keycode accessors. */

  int code(int column, int row)
  {
    return keycodes[row][column];
  }

  /* Common instance. */

  static Keypad_qi_lb60 keypad;
};

/* The actual definition. */

Keypad_qi_lb60 Keypad_qi_lb60::keypad;

/* Definition access. */

Keypad *Keypad::get_keypad()
{
  return &Keypad_qi_lb60::keypad;
}
