/*
 * LCD configuration value definitions for the JZ4740 and related SoCs.
 *
 * Copyright (C) 2018 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

enum Jz4740_lcd_config_bits
{
  Jz4740_lcd_config_lcd_pin             = 31, /* LCDPIN */
  Jz4740_lcd_config_tv_pal_halfline     = 30, /* JZ4780: TVEPEH */
  Jz4740_lcd_config_desc_8_word         = 28, /* JZ4780: NEWDES */
  Jz4740_lcd_config_tv_enable           = 26, /* JZ4780: TVEN */
  Jz4740_lcd_config_underrun_recover    = 25, /* JZ4780: RECOVER */
  Jz4740_lcd_config_ps_disable          = 23, /* PSM */
  Jz4740_lcd_config_cls_disable         = 22, /* CLSM */
  Jz4740_lcd_config_spl_disable         = 21, /* SPLM */
  Jz4740_lcd_config_rev_disable         = 20, /* REVM */
  Jz4740_lcd_config_hsync_mod_disable   = 19, /* HSYNM (hsync polarity choice) */
  Jz4740_lcd_config_pclock_mod_disable  = 18, /* PCLKM (dot/pixel clock polarity choice) */
  Jz4740_lcd_config_data_inverse        = 17, /* INVDAT (inverse output data) */
  Jz4740_lcd_config_sync_input          = 16, /* SYNDIR (hsync/vsync direction) */
  Jz4740_lcd_config_ps_reset            = 15, /* PSP */
  Jz4740_lcd_config_cls_reset           = 14, /* CLSP */
  Jz4740_lcd_config_spl_reset           = 13, /* SPLP */
  Jz4740_lcd_config_rev_reset           = 12, /* REVP */
  Jz4740_lcd_config_hsync_active_low    = 11, /* HSP (hsync polarity) */
  Jz4740_lcd_config_pclock_fall_edge    = 10, /* PCP (dot/pixel clock polarity) */
  Jz4740_lcd_config_de_active_low       = 9,  /* DEP (data enable polarity) */
  Jz4740_lcd_config_vsync_fall_edge     = 8,  /* VSP (vsync polarity) */
  Jz4740_lcd_config_bpp                 = 6,  /* 16/18/24bpp selection for generic TFT */
  Jz4740_lcd_config_stn_pins            = 4,  /* PDW (STN pins utilisation) */
  Jz4740_lcd_config_mode                = 0,
};

enum Jz4740_lcd_bpp_values
{
  Jz4740_lcd_bpp_16                     = 0,
  Jz4740_lcd_bpp_24                     = 1,  /* JZ4780 */
  Jz4740_lcd_bpp_18                     = 2,
};

enum Jz4740_lcd_modes
{ 
  Jz4740_lcd_mode_tft_generic           = 0,  /* parallel 16/18/24-bit panel */
  Jz4740_lcd_mode_tft_sharp             = 1,  
  Jz4740_lcd_mode_tft_casio             = 2,  
  Jz4740_lcd_mode_tft_samsung           = 3,  
  Jz4740_lcd_mode_ccir656_nonint        = 4,  /* non-interlaced (TV out) */
  Jz4740_lcd_mode_ccir656_int           = 6,  /* interlaced (TV out) */
  Jz4740_lcd_mode_stn_single_colour     = 8,  /* single == one panel */
  Jz4740_lcd_mode_stn_single_mono       = 9,  
  Jz4740_lcd_mode_stn_dual_colour       = 10, /* dual == two panels */
  Jz4740_lcd_mode_stn_dual_mono         = 11,
  Jz4740_lcd_mode_tft_serial            = 12, /* serial 8-bit panel */
  Jz4740_lcd_mode_lcm                   = 13, /* JZ4780 */
};

enum Jz4740_lcd_config_values
{
  Jz4740_lcd_de_positive                = 0 << Jz4740_lcd_config_de_active_low,
  Jz4740_lcd_de_negative                = 1 << Jz4740_lcd_config_de_active_low,
  Jz4740_lcd_pclock_positive            = 0 << Jz4740_lcd_config_pclock_fall_edge,
  Jz4740_lcd_pclock_negative            = 1 << Jz4740_lcd_config_pclock_fall_edge,
  Jz4740_lcd_hsync_positive             = 0 << Jz4740_lcd_config_hsync_active_low,
  Jz4740_lcd_hsync_negative             = 1 << Jz4740_lcd_config_hsync_active_low,
  Jz4740_lcd_vsync_positive             = 0 << Jz4740_lcd_config_vsync_fall_edge,
  Jz4740_lcd_vsync_negative             = 1 << Jz4740_lcd_config_vsync_fall_edge,
};
