/*
 * LCD panel definitions for the JZ4740 and related SoCs.
 *
 * Copyright (C) 2018 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include <stdint.h>

/* Panel configuration details. */

struct Jz4740_lcd_panel
{
  uint32_t config;                    /* LCD configuration */

  /* Display dimensions in pixels. */

  uint32_t width, height;
  uint32_t bpp;                       /* bits per pixel */

  /* Display timing information. */

  uint32_t frame_rate;                /* frame clock frequency (Hz) */

  /* Horizontal and vertical sync widths. */

  uint32_t hsync;                     /* in pixel clock periods */
  uint32_t vsync;                     /* in line periods */

  /* Inactive line region sizes before/after line data (in pixel clock periods). */

  uint32_t line_start, line_end;      /* similar to horizontal back porch, front porch */

  /* Inactive frame region sizes before/after frame data (in line periods). */

  uint32_t frame_start, frame_end;    /* vertical back porch, front porch */
};
