/*
 * Copyright (C) 2018 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include <l4/sys/types.h>
#include <stdint.h>



/* Value types. */

enum Jz4730_pwm_shutdown_mode
{
  Jz4730_pwm_shutdown_graceful = 0,
  Jz4730_pwm_shutdown_abrupt = 1,
};



#ifdef __cplusplus

#include <l4/devices/hw_register_block.h>

/* A simple abstraction for accessing the PWM registers. */

class Pwm_jz4730_chip
{
private:
  l4_addr_t _start, _end;
  Hw::Register_block<32> _regs;

public:
  Pwm_jz4730_chip(l4_addr_t start, l4_addr_t end);

  void disable();
  void enable();
  void set_shutdown_mode(enum Jz4730_pwm_shutdown_mode mode);

  void set_control(uint8_t control);
  void set_duty(uint16_t duty);
  void set_period(uint16_t period);
};

#endif /* __cplusplus */



/* C language interface. */

EXTERN_C_BEGIN

void *jz4730_pwm_init(l4_addr_t start, l4_addr_t end);

void jz4730_pwm_disable(void *pwm);
void jz4730_pwm_enable(void *pwm);
void jz4730_pwm_set_shutdown_mode(void *pwm, enum Jz4730_pwm_shutdown_mode mode);

void jz4730_pwm_set_control(void *pwm, uint8_t control);
void jz4730_pwm_set_duty(void *pwm, uint16_t duty);
void jz4730_pwm_set_period(void *pwm, uint16_t period);

EXTERN_C_END
