/*
 * PWM client library to access PWM servers.
 *
 * Copyright (C) 2018 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <l4/cxx/ipc_stream>
#include <l4/re/env>

#include "pwm-client.h"

/* PWM device interface. */

int
Pwm_device_interface::disable() throw()
{
  L4::Ipc::Iostream s(l4_utcb());

  return l4_error(s.call(cap(), Pwm_op_disable));
}

int
Pwm_device_interface::enable() throw()
{
  L4::Ipc::Iostream s(l4_utcb());

  return l4_error(s.call(cap(), Pwm_op_enable));
}

int
Pwm_device_interface::set_control(int control) throw()
{
  L4::Ipc::Iostream s(l4_utcb());

  s << control;
  return l4_error(s.call(cap(), Pwm_op_set_control));
}

int
Pwm_device_interface::set_duty(int duty) throw()
{
  L4::Ipc::Iostream s(l4_utcb());

  s << duty;
  return l4_error(s.call(cap(), Pwm_op_set_duty));
}

int
Pwm_device_interface::set_period(int period) throw()
{
  L4::Ipc::Iostream s(l4_utcb());

  s << period;
  return l4_error(s.call(cap(), Pwm_op_set_period));
}
