/*
 * Data space allocation utility functions.
 *
 * Copyright (C) 2018 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <l4/re/dataspace>
#include <l4/re/env>
#include <l4/re/rm>
#include <l4/re/util/cap_alloc>

#include "dataspace.h"

/*
Allocate and map a region containing data, using the given mem_size to indicate
the required region size, along with a pointer to be set to the allocated
region. Return a capability which may be invalid if allocation or mapping
failed.
*/

L4::Cap<L4Re::Dataspace> allocate_data(l4_size_t mem_size, void **allocated)
{
  /* Obtain a capability for the allocated memory. */

  L4::Cap<L4Re::Dataspace> mem = L4Re::Util::cap_alloc.alloc<L4Re::Dataspace>();

  if (!mem.is_valid())
    return L4::Cap<L4Re::Dataspace>::Invalid;

  /* Allocate memory for the data. */

  if (L4Re::Env::env()->mem_alloc()->alloc(mem_size, mem, L4Re::Mem_alloc::Continuous))
  {
    L4Re::Util::cap_alloc.free(mem);
    return L4::Cap<L4Re::Dataspace>::Invalid;
  }

  /* Map the memory to a virtual address. */

  if (L4Re::Env::env()->rm()->attach(allocated, mem->size(), L4Re::Rm::Search_addr,
                                     L4::Ipc::make_cap_rw(mem)))
  {
    L4Re::Util::cap_alloc.free(mem);
    return L4::Cap<L4Re::Dataspace>::Invalid;
  }

  return mem;
}
