/*
 * Dynamic loading support.
 *
 * Copyright (C) 2018 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <l4/devices/dl.h>
#include <dlfcn.h>
#include <stdio.h>

void *load_function(const char *filename, const char *function)
{
  char libname[64];
  FILE *f = fopen(filename, "r");
  size_t nread;
  void *handle;

  /* Obtain the library name from the file. */

  if (f == NULL) return 0;

  nread = fread(libname, sizeof(char), 63, f);
  fclose(f);

  if (ferror(f)) return 0;

  libname[nread] = '\0';

  /* Obtain the linked data. */

  handle = dlopen(libname, RTLD_NOW);
  if (handle == NULL) return 0;

  return dlsym(handle, function);
}
