/*
 * Access the LCD on JZ4740-based platforms.
 *
 * Copyright (C) 2018 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <l4/devices/lcd-device.h>
#include <l4/re/c/video/view.h>
#include <l4/sys/cache.h>

#include <stdint.h>

int main(void)
{
  l4re_video_view_info_t view_info;
  l4_size_t i, limit, line, digit, row, mask, value, onpix, offpix;

  /* Obtain device and get framebuffer details. */

  void *lcd_device = lcd_get_device();
  l4_addr_t fb_vaddr = lcd_get_framebuffer(lcd_device);
  l4_size_t fb_size = lcd_get_framebuffer_size(lcd_device);
  uint32_t *fb32 = (uint32_t *) fb_vaddr;
  uint16_t *fb16 = (uint16_t *) fb_vaddr;

  if (!fb_vaddr) return 1;

  lcd_get_view_info(lcd_device, &view_info);
  lcd_enable(lcd_device);

  /* Try and show some values. */

  onpix = 0xffaaffaa; offpix = 0x11551155;
  mask = 0x80000000; value = (l4_size_t) view_info.width;

  i = 0;
  limit = fb_size / view_info.pixel_info.bytes_per_pixel;
  line = limit / view_info.height;
  digit = line / 32;
  row = line * 10;

  while (i < limit)
  {
    if (view_info.pixel_info.bytes_per_pixel == 2)
      fb16[i] = value & mask ? onpix : offpix;
    else
      fb32[i] = value & mask ? onpix : offpix;

    i++;

    if ((i % digit) == 0)
    {
      if (mask == 1) mask = 0x80000000;
      else mask >>= 1;

      onpix = (onpix >> 8) | ((onpix & 0xff) << 24);
      offpix = (offpix >> 8) | ((offpix & 0xff) << 24);

      if (i == row) value = (l4_size_t) view_info.height;
      else if (i == (row * 2)) value = 0x80000001;
    }
  }

  l4_cache_clean_data((unsigned long) fb_vaddr, (unsigned long) fb_vaddr + fb_size);

  while (1);

  return 0;
}
