/*
 * Memory allocation utility functions.
 *
 * Copyright (C) 2018 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <l4/io/io.h>
#include <l4/re/env.h>
#include <l4/re/c/mem_alloc.h>
#include <l4/re/c/util/cap_alloc.h>
#include <l4/util/util.h>
#include <l4/vbus/vbus.h>

#include "memory.h"

int get_device(char const *hid, l4io_device_handle_t *dh, l4io_resource_handle_t *rh)
{
  return l4io_lookup_device(hid, dh, 0, rh);
}

int get_resource(l4io_device_handle_t dh, l4io_resource_t *res,
                             enum l4io_resource_types_t type)
{
  int current = 0, result = 0;
  l4_cap_idx_t vbus = l4re_env_get_cap("vbus");

  do
  {
    result = l4vbus_get_resource(vbus, dh, current, res);
    current++;
  }
  while ((!result) && (res->type != type));

  return result;
}

int get_memory(char const *hid, l4_addr_t *start, l4_addr_t *end)
{
  l4io_device_handle_t dh;
  l4io_resource_handle_t rh;
  l4io_resource_t res;
  int result;

  result = get_device(hid, &dh, &rh);

  if (result < 0)
    return result;

  result = get_resource(dh, &res, L4IO_RESOURCE_MEM);

  if (result)
    return result;

  result = l4io_request_iomem(res.start, res.end - res.start + 1,
                              L4IO_MEM_NONCACHED, start);

  if (result)
    return result;

  *end = *start + (res.end - res.start + 1);

  return 0;
}
